% A program to show the use of coherence measures
% It demonstrates that a window of RANDOM samples is indeed incoherent,
% and in fact measures the mean and standard deviation of ring pairs.

%close all;
clear all;
Nsamples = 8;

% make a series of test windows
n = 5;
N = 2*n+1;
[x,y] = meshgrid( -n:n );
y = flipud(y);
r = sqrt( x.^2 + y.^2 );
a = atan2( y,x );

W{1} = zeros(N,N);  % a flat window
W{2} = x >= 0; % a perfect edge
W{3} = (x <= 1) & (x >= -1); %  a perfect ridge 
W{4} =  (x  >= 0) & (y >= 0); % A perfect corner
W{5} = r > 2; % a perfect ring
aa = r/max(max(r))*pi/6;
W{6} = abs(a+aa) > pi/4 & abs(a+aa) < 3*pi/4; % swirl
clear aa;
for k = 7:107
  W{k} = rand(N,N); % random window
end


pict = zeros(N,N,3); % a template for a small picture
[i,j] = find( (x==0) & (y==0) ); % location of origin in pucture
[x,y] = make_samples( Nsamples ); % samples on unit ring
rad = 0:n;
dd = zeros( length(rad) );
mask = tril( ones(size(dd)), -1 );
Nmask = 1; %Nsamples^2; %sum(sum(mask));
for k = 1:size(W,2)
  
  % make a picture and sample it to make a circular window
  pict(:,:,1) = W{k};
  pict(:,:,2) = W{k};
  pict(:,:,3) = W{k};
  w = location2window( pict, [i;j], [x;y], rad );
  
  % compute the Euclidean distance between each pair of rings
  for r1 = 1:length(rad)
    for r2 = 1:length(rad)
      d = w(r1,:,:) - w(r2,:,:);
      dd(r1,r2) = sqrt( sum(sum(d.^2,3)) /Nsamples );
    end
  end
  K{k} =dd; 
 
  h = K{k}(mask == 1);
  sh = sum(h);
  if sh < eps^2
    ent(k) = 0;
  else
      h = h(h > eps^2) / sh;
      ent(k) = sum( h.*log(h) );
  end
end

figure;
hold on;
for k = 1:6
  b = K{k}(mask==1);
  b = b / Nmask;
  plot( b );
end
z = [];
for k = 7:size(W,2)
  b = K{k}(mask==1);
  b = b / Nmask;
  z = [z b];
  plot( b,'g');
end
zm = mean( z, 2 );
zs = std( z,[],2 );
plot( zm, 'k' );
plot( zm+zs, 'k' );
plot( zm-zs, 'k' );

return;

coherentdata.mean= zm;
coherentdata.stdev = zs;

