function featuredata = training2coherence_measure( trainingdata, Nsample)

% featuredata = training2coherence_measure( trainingdata, Nsample )
% Make featuredata from the training data using Nsample around each ring.

% Record sampling information
featuredata.Nsample = Nsample;

% make and record sample locations on the unit circle
[samplelocation(1,:), samplelocation(2,:)] = make_samples( Nsample );
featuredata.samplelocation = samplelocation;

% count the number of feature types
Ntype = size( trainingdata.typename, 2);
featuredata.Ntype = Ntype;

featuredata.type = [];
featuredata.rad = [];
featuredata.feature = [];

% collect a set of features for each type from the training data
for i = 1:size( trainingdata.session,2)
  pict = pict_read( trainingdata.session{i}.fullname ); % read in the picture
  rad = trainingdata.session{i}.rad;
  for j = 1:Ntype
    location = trainingdata.session{i}.location{j};
    for k = 1:size(location,2)
      % get a window
      w = location2window( pict, location(:,k), samplelocation, 1:rad );
      % measure its coherence
      m = coherencemeasure_window( w, [] );
      
      % record data about the feature
      featuredata.type(end+1) = j;
      featuredata.rad(end+1) = rad;
      featuredata.feature(:,end+1) = m;
    end % for k
  end % for j
end % for i

return;
