function jnddata = jnd_include_sample( jnddata, p, f )

% jnddata = jnd_grow( jnddata, p, f )
% Add the sample (p,f) to the jnd data.
% If the input point is very close to an existing sample, that sample is update.
% Otherwise the point is inserted by growing the Octree.

index = find( dist2( p', jnddata.point' ) < eps*eps );
if ~isempty( index )
  % sample exists
  jnddata = jnd_sample_update( jnddata, f );
  return;
end

% sample needs inserting into tree

% decide which leaf the new sample is in
inode = jnd_locate_leaf( p, jnddata );

% make new points that divide the leaf up
index = Octree_getdata( jnddata, inode );   % index into the samples
minp = min( jnddata.point(:,index), [], 2 );% min and...
maxp = max( jnddata.point(:,index), [], 2 ); %...max extremities
newpoint = zeros(3,27); % open a buffer
m = 0;
q = [minp p maxp];
for k = 1:3
  for j = 1:3
    for i = 1:3
      m = m+1;
      newpoint(:,m) = [q(1,i);q(2,j);q(3,k)];
    end
  end
end
newfunc = zeros( 3, 27 ); % make a parallel array of function values

% decide which of the new points already is a sample point
% This is decides using a small Euclidean tolerance between each new point
% and existing samples. The eps*eps is used becasue squared distances are used.
[in,inindex] = find( dist2( newpoint', jnddata.point' ) < eps*eps );

% by construction the 14th new point is the point supplied
% we don't want to repeatedly add the same point
if ismember(14,in)
  return;
end

% obtain a list of points that are not amogst the existing samples
notin = setdiff( 1:27, in );

ndata = size( jnddata.point, 2 );
jnddata.point = [jnddata.point newpoint(:,notin)]; % append new points
jnddata.func = [jnddata.func newfunc(:,notin)]; % append new function values

newpointi = [1:size(notin,2)] + ndata; % and index them

% estimate a value for each of the new sample points
% the exception is the supplied point, which has a function value supplied
% the supplied point has local index == 14 by construction
for j = 1:length(notin)
  if notin(j) == 14
    jnddata.func(:,ndata+j) = [mean(f),std(f),1]; % we assume f has been measured
  else
    k = newpointi(j);
    pos = newpoint(:,notin(j));
    jnddata.func(1:2,k) = jnd_estimate( newpoint(:,notin(j)) , jnddata );
    % jnddata.func(2,k) = 0;
    jnddata.func(3,k) = 0;
  end
end

% and compute abolute indices into the point and function arrays
% for each new leaf
mdata = zeros(8,8);
for i = 1:length(in)
  mask = (jnddata.subcubeindex == in(i));
  mdata = mdata + mask*inindex(i);
end
for i = 1:length(notin)
  mask = (jnddata.subcubeindex == notin(i));
  mdata = mdata + mask*newpointi(i);
end

% make a data item for each leaf.
% A data structure is required by the Octree library
for i = 1:8
  data{i} = mdata(i,:);
end

% and finally frwo the tree downward.
jnddata = Octree_growdown( jnddata, inode, data );

return;
