function jnddata = jnd_init()

% jnddata = jnd_init()
% Inittialise jnd data model.

jnddata = Octree_init; % an empty model

% add one node - the root
[jnddata,root] = Octree_newnode( jnddata); % make the root

jnddata.root = root; % record the root value for ease of use later

% A pair of parallel arrays, point, and func. Given a valid index, i,
% a sample comprises  the pair point(:,i), func(:,i).

jnddata.point = [ % where the sample is
  [0 0 0]; % left  bottom near
  [1 0 0]; % right bottom near
  [0 1 0]; % left  top    near
  [1 1 0]; % right top    near
  [0 0 1]; % left  bottom far
  [1 0 1]; % right bottom far
  [0 1 1]; % left  top    far
  [1 1 1]; % right top    far
]';

jnddata.func = [ % what the sample is
  ones(1,8);  % mean value
  zeros(1,8); % std value. If zero, then no trial
  zeros(1,8)  % number of trials. If zero then NO trial
];


% store the indicies of the sample points
jnddata = Octree_setdata( jnddata, root, 1:8 );

% Define an ordering of points on the vertex of a cube.
% This helps ensure consistency across all functions
jnddata.ordering = [
  [0 0 0]; [1 0 0]; [0 1 0]; [1 1 0];
  [0 0 1]; [1 0 1]; [0 1 1]; [1 1 1];
]';

% Define an ordering of vertex when the cube is partioned into
% eight subcubes. This ordering is consistent with the ordering just given.
% Neither ordering gives an absolute index in the list of samples, but
% does help such indices to be computed.
jnddata.subcubeindex = [
  [1 2 4 5 10 11 13 14];
  [2 3 5 6 11 12 14 15];
  [4 5 7 8 13 14 16 17];
  [5 6 8 9 14 15 17 18];
  [10 11 13 14 19 20 22 23];
  [11 12 14 15 20 21 23 24];
  [13 14 16 17 22 23 25 26];
  [14 15 17 18 23 24 26 27]
];



return;
