function [pict,flag] = jnd_make_pict( pos, jnd, Nedge )

% jnd = jnd_make_pict( pos, jnd )
% Make a picture with colour centre pos, radius jnd


% specify planes
planepos = [ [0 0  0]; [0  0 0]; [ 0 0 0]; [1 1 1]; [1 1 1]; [1 1 1] ]';
planenrm = [ [0 0 -1]; [0 -1 0]; [-1 0 0]; [0 0 1]; [0 1 0]; [1 0 0] ]';

% find the largest possible radius
% otherwise we might not get a picture at all
corner = [
  [0 0 0];
  [1 0 0];
  [0 1 0];
  [1 1 0];
  [0 0 1];
  [1 0 1];
  [0 1 0];
  [1 1 1];
]';
maxr = max( dist2( pos', corner' ) );
if jnd*jnd > maxr
  jnd = sqrt(maxr);
end

N = Nedge*Nedge; % number of points wanted
q = []; % no points so far

% generate samples on the surface of a sphere INSIDE the RGB cube
iter = 0;
Niter = 100;
while N > 0
  iter = iter+1;
  if iter > Niter
    break;
  end
  qq = Nsamples( pos, jnd, N );
  mask = zeros( 1, size(qq,2) );
  for i = 1:6
    % detect which points are outside this plane
    d = (qq - repmat( planepos(:,i),1,N))' * planenrm(:,i);
    mask = mask | (d' > 0);
  end
  mask = ~mask;
  q = [q qq(:, mask )];
  N = N - sum(mask);
end

% if q has less than Nedge^2 entries
% fill in remaining positions with projected colours
% and warn the user
if N > 0
  qq(1,:) = min( qq(1,:), 1 );
  qq(2,:) = min( qq(2,:), 1 );
  qq(3,:) = min( qq(3,:), 1 );
  qq(1,:) = max( qq(1,:), 0 );
  qq(2,:) = max( qq(2,:), 0 );
  qq(3,:) = max( qq(3,:), 0 );
  q = [q qq];
  q = q(:, randperm(Nedge*Nedge) );
  flag = 1;
else
  flag = 0;
end



% now make up and display the picture
M = 2*Nedge+1;
pict = ones( M,M,3 );
pict(:,:,1) = pos(1)*pict(:,:,1);
pict(:,:,2) = pos(2)*pict(:,:,2);
pict(:,:,3) = pos(3)*pict(:,:,3);

k = 0;
for i = 2:2:M
  for j= 2:2:M
    k = k+1;
    pict(i,j,1) = q(1,k);
    pict(i,j,2) = q(2,k);
    pict(i,j,3) = q(3,k);
  end
end

if any(any(any( (pict < 0) | (pict >1) )))
  save 'jnd_make_pict.mat';
end

return;

%%%%%%%%%%%%%%%%%%%%%%%%%

function q = Nsamples( pos, r, N )

q = gsamp( [0 0 0], eye(3), N);      % generate Gaussian samples
q = q - repmat( mean(q,1), N,1 );    % make sure they zero centred
d = dist2( [0 0 0], q );             % and have unit distance from the origin

mask = d < eps;
M = sum( mask );
while M > 0
  qq = gsamp( [0 0 0], eye(3), M );
  qq = qq - repmat( mean(q,1), M, 1 );
  d(mask) = dist2( [0 0 0], qq );
  mask = d < eps;
  M = sum( mask );
end

q = q' ./ repmat( sqrt(d), 3, 1 );   % Note the use of q' here!
q = q*r + repmat(pos,1,N); % scale and then shift the points

return;
