function [vpict,jpict] = pict_prob_visible( pict, samp, scale, jpict )

% vpict = pict_prob_visible( pict, samp, scale, jpict )
% Return the probability map for 'visibility' ,
% at given scale, using sample locations given in samp.
% jpict is anoptional argument, the jnd at each pixel.



N = size( pict );

% make a jnd map; jpict is an optional argument
if nargin < 4
  jpict = jnd_pict( pict );
end

%load jnddata;
%ndmax = jnd_distance( [0;0;0], [1;1;1], jnddata );

Nring = scale;
Nsamp = size( samp, 2 );

% the numbers mu and sigma have been empirically determined
% to fit to 'visibility' data from training over discs
mu = 1;
sigma = 0.2;

h = waitbar(0,'pict\_prob\_visible');
vpict = zeros( N(1), N(2) );
for j = (scale+1):(N(1)-scale)
  waitbar( j/N(1), h );
  for i = (scale+1):(N(2)-scale)
       % fprintf('%d %d\n', i,j );
    % get a window
    w = location2window( pict, [i;j], samp, 1:scale ); 
    % get its differential structure
    [da,dr] = window_radiff( w, pict(j,i,:) ); % pict(j,i,:) is central colour
    % compute the magnitude of the total differential
    dc = sqrt( sum(  (da + dr).^2, 3 ) );
    % sample jnd map at same locations
    jnd = location2window( jpict, [i,j], samp, 1:scale );
    % compute probability that any given transition is visible.
    
%    p = (erf( (dc - jnd(:,:,1)) ./ jnd(:,:,2)) + 1)/2;
 %   vpict(j,i) = mean( max(p,[],2) );
    
    d = mean(mean( dc ./ jnd(:,:,1) ));
    vpict(j,i)  = 0.5*(1+erf( (d-mu)/sigma )); 
  end
end
close( h );

return;
