function [window,flag] = location2window( pict, location, samplelocation, rad )

% window = location2window( pict, location, samplelocation, rad )
% Collect full colour samples from pict around the give location at
% in a disc as points specified by samplelocations and rad.
% Assume all locations lie within the pictuer area.

% number of samples around each ring
Nsample = size(samplelocation,2);

% number of rings
Nrad = size(rad,2);

% make a window buffer
window = zeros(Nrad,Nsample,size(pict,3));

% use nearest neighbour sampling
for j = 1:Nrad
  u = round( rad(j)*samplelocation(1,:) + location(1) );
  v = round( rad(j)*samplelocation(2,:) + location(2) );
  for i = 1:Nsample
    window(j,i,:) = pict( v(:,i), u(:,i), : );
  end
end


return;
% 
% N = size(pict); size of picture
% flag = 1; % assume valid sample
% for i = 1:Nrad
%   u = round( location(1) + rad(i)*samplelocation(1,:));
%   v  = round(location(2) + rad(i)*samplelocation(2,:));
%   if any(u < 1) | any(v < 1 ) | any(u > N(2)) |  any(v > N(1))
%     flag = 0;
%     return;
%   end
%   for j = 1:Nsample
%     window(i,j,:) = pict( v(:,j),u(:,j), : );
%   end
% end

return;


% Running the test below shows
%   t1 = 0.06 (nearest neigbour)
%   t2 = 50.452 (bilinear interpolation using in-built MATLAB)
%   t3 = 11.0860 (bilinear interpolation using C code)
% The errors using the MATLAB result as standard are computed as the max diff
%  between the windows
%    e1 = 0.5340
%    e2 = 0 (the comparator)
%    e3 = 2.8e-15
% Unless rounding makes for very big classification errors (no evidence so far it does)
% Then the alternatives are not good value.

% t1 = cputime;
% for trial = 1:100
% for i = 1:Nrad
%   u = round( location(1) + rad(i)*samplelocation(1,:));
%   v  = round(location(2) + rad(i)*samplelocation(2,:));
%   for j = 1:Nsample
%     window(i,j,:) = pict( u(:,j),v(:,j), : );
%   end
% end
% end
% t1 = cputime-t1;
% window1 = window;
% 
% % fill the window with colour - interp2 is VERY expensive
% t2 = cputime;
% for trial = 1:100
% for i = 1:Nrad
%   u = location(1) + rad(i)*samplelocation(1,:);
%   v  = location(2) + rad(i)*samplelocation(2,:);
%   window(i,:,1) = interp2( pict(:,:,1), u, v );
%   window(i,:,2) = interp2( pict(:,:,2), u, v );
%   window(i,:,3) = interp2( pict(:,:,3), u, v );
% end
% end
% t2 = cputime-t2;
% window2 = window;
% 
% 
% % fill the window with colour - interp2 is VERY expensive
% t3 = cputime;
% for trial = 1:100
% for i = 1:Nrad
%   u = location(1) + rad(i)*samplelocation(1,:);
%   v  = location(2) + rad(i)*samplelocation(2,:);
%   window(i,:,1) = pictInterp( pict(:,:,1), u, v );
%   window(i,:,2) = pictInterp( pict(:,:,2), u, v );
%   window(i,:,3) = pictInterp( pict(:,:,3), u, v );
% end
% end
% t3 = cputime-t3;
% window3 = window;
% % 
% % flag = ~any(any(any( isnan(window) ))); % flag == 1 for a valid sample
% 
% return;
