function mask = mask_new( mask, scale )

% mask = mask_new( mask, scale )
% make a new mask for differeniation etc at the given scale scale
% The mask returned is in mask{scale}, and comprises
% mask.raw - a binary map
% mask.dx - dg/dx (g is a gaussian that fits the window)
% mask.dy - dg/dy (g is the same gaussian)
% these is one field for each colour channel (which helps provide succinct convolution over colour images)
[x,y] = meshgrid( -scale:scale );
z = x.^2 + y.^2;
mask{scale}.raw = z <= scale^2; % basic mask
sigma = scale/3;
z = exp( -0.5 * z / sigma^2 ).*mask{scale}.raw;
mask{scale}.dx = -(x/sigma).*z; % df/dx
mask{scale}.dx = mask{scale}.dx / max(max( mask{scale}.dx ));
mask{scale}.dy = -(y/sigma).*z; % df/dy
mask{scale}.dy = mask{scale}.dy / max(max( mask{scale}.dy ));

% copy masks to other colour channels
mask{scale}.raw(:,:,2) = mask{scale}.raw;
mask{scale}.raw(:,:,3) = mask{scale}.raw(:,:,1);
mask{scale}.dx(:,:,2) = mask{scale}.dx;
mask{scale}.dx(:,:,3) = mask{scale}.dx(:,:,1);
mask{scale}.dy(:,:,2) = mask{scale}.dy;
mask{scale}.dy(:,:,3) = mask{scale}.dy(:,:,1);

return;

        