function rpict = rare_prob_pict( pict, samplelocation, rad)

%  rpict = rpict_prob_pict( pict )
% Compute a 'rare' proobablity picture given an input

rmax = max(rad);
Nsample = size( samplelocation,2 );
N = size( pict );
Ndim = 3*Nsample*(1+rmax); % dimension of each feature vector

% make an eigenmodel
emodel = EigModel_make( [] );
data = zeros( Ndim, (N(2)-rmax) - (rmax+1) +2 );
h = waitbar(0,'rare\_prob\_pict - part 1');
for j = (rmax+1):(N(1)-rmax)
  waitbar( j/N(1),h);
  for i =(rmax+1):(N(2)-rmax)
    w = location2window( pict, [i,j], samplelocation, rad ); % a window
    data(:,i-rmax+1) = w(:); % a 'feature'
  end % for i
  emodel = EigModel_add(  emodel,  EigModel_make( data ) );
end % for j
close(h);

clear data;
emodel = EigModel_deflate( emodel, 'keepf', 0.99 );

% compute Mahalanobis distance
% if eigmodel is replaces, then likelihood may be better - with requisite changes at (a) and (b) below
h = waitbar(0,'rare\_prob\_pict - part 2');
rpict = zeros( N(1), N(2) );
for j = (rmax+1):(N(1)-rmax)
  waitbar( j/N(1), h );
  for i =(rmax+1):(N(2)-rmax)
    w = location2window( pict, [i,j], samplelocation, rad ); % a window
    rpict(j,i) =  EigModel_Mahalanobis( w(:), emodel );
  end % for i
end % for j
close(h);

% sort the distances, and associated locations
m = rpict( (rmax+1):(N(1)-rmax), (rmax+1):(N(2)-rmax) );
[x,y] = meshgrid( (rmax+1):(N(2)-rmax), (rmax+1):(N(1)-rmax) );
m = sortrows( [m(:), x(:), y(:)] );
ms = sum( m(:,1) );
m(:,1) = m(:,1)/ms;

% estimate rarity as a cumulative probability
rpict = zeros(N(1),N(2));
i = 1;
p = 0; % prob of being rare
h = waitbar(0,'rare\_prob\_pict - part 3');
while i <= size(m,1)
  waitbar( i/size(m,1), h );
  current = m(i,1);
  accum = p;
  while i <= size(m,1) & m(i,1) == current
    rpict( m(i,3),m(i,2) ) = p;
    accum = accum + m(i,1);
    i = i+1;
  end
  p = accum;
end
close(h );

return;

% estimate rarity as a cumulative probability
for i = 1:size(m,1)
  mask = m(:,1) <= m(i,1); % (a) identify all locations with smaller M-distance
  rpict( m(i,3), m(i,2) ) = sum( m(mask,1) ) / ms;
end

return;

