% A program to estimate the probability density of a set of windows over a picture.
close all;
clear all;

% input a picture
% define a set of pixel-centred patterns
% model probaiilty distribution of patterns
% compute probability a pattern about a pixel is rare, associate this with centre pixel
% output rarity map

% input a picture
pict = getpict( 4 );
imshow( pict );

N  = size( pict );
s= [1 2 3];
rpict = zeros( N(1),N(2), length(s) );
  
for scale = s
  
  M = (N(1)-2*scale)*(N(2)-2*scale);
  m = 3*(2*scale+1)^2;
  
  t1(scale) = cputime;
  
  g = zeros( m, M );
  k = 0;
  for j = (scale+1):(N(1)-scale)
    for i = (scale+1):(N(2)-scale)
      k = k+1;
      f = pict( j-scale:j+scale, i-scale:i+scale, : );
      g(:,k) = f(:);
    end
  end
  e = EigModel_make( g );
  
  t2(scale) = cputime;
  
  g = zeros( e.N, 3 );
  k = 0;
  for j = (scale+1):(N(1)-scale)
    for i = (scale+1):(N(2)-scale)
      k = k+1;
      f = pict( j-scale:j+scale, i-scale:i+scale, : );
      h = EigModel_Mahalanobis( f(:), e );
      g( k,1 ) = h;
      g( k,2:3 ) = [i,j];
    end
  end
  g = flipud( sortrows( g ) );
  
  t3(scale) = cputime;
  

  H = sum( g(:,1) );
  h = H;
  for k = 1:size( g, 1 );
    i = g(k,2);
    j = g(k,3);
    rpict( j,i, scale ) = h/H;
    h = h - g(k,1);
  end
  
  t4(scale) = cputime;
  figure;
  imshow( rpict(:,:,scale) );
end

return;

%pict = getpict(1);
pict = pict_make_special( [128,128] );

rad = 3;
Nsample = 16;
[x,y] = make_samples( Nsample );

rpict = rpict_prob_pict( pict, [x;y], rad );

return;

N = size( pict );
Ndim = 3*Nsample; % dimension of each feature vector

% make an eigenmodel
emodel = EigModel_make( [] );
data = zeros( Ndim, (N(2)-rad) - (rad+1) +2 );
for j = (rad+1):(N(1)-rad)
  for i =(rad+1):(N(2)-rad)
    w = location2window( pict, [i,j], [x;y], rad ); % a window
    data(:,i-rad+1) = w(:); % a 'feature'
  end % for i
  emodel = EigModel_add(  emodel,  EigModel_make( data ) );
end % for j

clear data;
emodel = EigModel_deflate( emodel, 'keepf', 0.99 );

% compute Mahalanobis distance
rpict = zeros( N(1), N(2) );
for j = (rad+1):(N(1)-rad)
  for i =(rad+1):(N(2)-rad)
    w = location2window( pict, [i,j], [x;y], rad ); % a window
    rpict(j,i) =  (1+erf( EigModel_Mahalanobis( w(:), emodel )/3 - 3 ))/2;
  end % for i
end % for j



