clear all;
close all;

if 0
Nsample = 16;

load 'trainingdata.mat';

fpictname = [trainingdata.pictpath trainingdata.pictname{1}];
hpictname = [trainingdata.pictpath trainingdata.pictname{7}];

% function featuredata = type_feature_data_make( trainingdata, Nsample)
ffeaturedata.Nsample = Nsample; % Record sampling information
[samplelocation(1,:), samplelocation(2,:)] = make_samples( Nsample ); % make and record sample locations on the unit circle
ffeaturedata.samplelocation = samplelocation;
Ntype = size( trainingdata.typename, 2); % count the number of feature types
ffeaturedata.Ntype = Ntype;
ffeaturedata.type = [];
ffeaturedata.rad = [];
ffeaturedata.feature = [];

hfeaturedata.Nsample = Nsample; % Record sampling information
[samplelocation(1,:), samplelocation(2,:)] = make_samples( Nsample ); % make and record sample locations on the unit circle
hfeaturedata.samplelocation = samplelocation;
Ntype = size( trainingdata.typename, 2); % count the number of feature types
hfeaturedata.Ntype = Ntype;
hfeaturedata.type = [];
hfeaturedata.rad = [];
hfeaturedata.feature = [];

% collect a set of features for each type from the training data
for i = 1:size( trainingdata.session,2)
  
  pict = pict_read( trainingdata.session{i}.fullname ); % read in the picture
  jpict = type_jpict4pict( trainingdata.session{i}.fullname ); % acquire a jnd map
  rad = trainingdata.session{i}.rad;
  
  for j = 1:Ntype
    location = trainingdata.session{i}.location{j};
    for k = 1:size(location,2)     
      % get a window
      w = location2window( pict, location(:,k), samplelocation, 1:rad );
      % get a jnd window too
      jw = location2window( jpict, location(:,k), samplelocation, 1:rad );
      % differential structure
      [da,dr] = window_radiff( w, pict(location(2,k),location(1,k),:) );     
      % turn it into a feature
      f = type_window2feature( da, dr , jw, 1:rad  );
      % record data about the feature
      
      if strcmp( trainingdata.session{i}.fullname, fpictname ) == 1
        ffeaturedata.type(end+1) = j;
        ffeaturedata.rad(end+1) = rad;
        ffeaturedata.feature(:,end+1) = f;
      elseif strcmp( trainingdata.session{i}.fullname, hpictname ) == 1
        hfeaturedata.type(end+1) = j;
        hfeaturedata.rad(end+1) = rad;
        hfeaturedata.feature(:,end+1) = f;
      end
      
    end % for k
  end % for j
end % for i

% classifiers
fclassifier = type_classifier_make( ffeaturedata, rad );
hclassifier = type_classifier_make( hfeaturedata, rad );

% add the classifiers
classifier = [];
classifier.Nsample = fclassifier.Nsample;
classifier.rad = fclassifier.rad;
classifier.samplelocation  = fclassifier.samplelocation;
classifier.Ntype = fclassifier.Ntype;

for i = 1:classifier.Ntype
  classifier.prior(i) = fclassifier.gmm{i}.Nd + hclassifier.gmm{i}.Nd;
  gmm = { fclassifier.gmm{i} hclassifier.gmm{i} };
  [dumA, k, dumC ] = GMM_addN( gmm, 1 );
  classifier.gmm{i} = dumA{k};
end

[samp(1,:), samp(2,:)] = make_samples( Nsample );

load coherencedata;

fpict = pict_read( fpictname );
fjpict = jnd_pict( fpict );
fvpict = pict_prob_visible( fpict, samp, rad, fjpict );
fcpict = coherence_prob_pict( fpict, samp, rad+2, coherencedata );
ffkpict = type_prob_pict( fpict, fclassifier, rad, fjpict ); % face with face classifer
fkpict = type_prob_pict( fpict, classifier, rad, fjpict ); % face with combined classifer

hpict = pict_read( hpictname );
hjpict = jnd_pict( hpict );
hvpict = pict_prob_visible( hpict, samp, rad, hjpict );
hcpict  = coherence_prob_pict( hpict, samp, rad+2, coherencedata );
hfkpict = type_prob_pict( hpict, fclassifier, rad, hjpict ); % houses with face classifer
hhkpict = type_prob_pict( hpict, hclassifier, rad, hjpict ); % houses with house classifer
hkpict = type_prob_pict( hpict, classifier, rad, hjpict ); % houses with combined classifier

ffkvpict(:,:,1) = ffkpict(:,:,1).*(1-fvpict) .* fcpict;
ffkvpict(:,:,2) = ffkpict(:,:,2) .* fvpict .* fcpict;
ffkvpict(:,:,3) = ffkpict(:,:,3) .* fvpict .* fcpict;
ffkvpict(:,:,4) = ffkpict(:,:,4) .* fvpict .* fcpict;
ffkvpict(:,:,5) = ffkpict(:,:,5) .* fvpict .* fcpict;

fkvpict(:,:,1) = fkpict(:,:,1).*(1-fvpict) .* fcpict;
fkvpict(:,:,2) = fkpict(:,:,2) .* fvpict .* fcpict;
fkvpict(:,:,3) = fkpict(:,:,3) .* fvpict .* fcpict;
fkvpict(:,:,4) = fkpict(:,:,4) .* fvpict .* fcpict;
fkvpict(:,:,5) = fkpict(:,:,5) .* fvpict .* fcpict;


hfkvpict(:,:,1) = hfkpict(:,:,1).*(1-hvpict) .* hcpict;
hfkvpict(:,:,2) = hfkpict(:,:,2) .* hvpict .* hcpict;
hfkvpict(:,:,3) = hfkpict(:,:,3) .* hvpict .* hcpict;
hfkvpict(:,:,4) = hfkpict(:,:,4) .* hvpict .* hcpict;
hfkvpict(:,:,5) = hfkpict(:,:,5) .* hvpict .* hcpict;

hhkvpict(:,:,1) = hhkpict(:,:,1).*(1-hvpict) .* hcpict;
hhkvpict(:,:,2) = hhkpict(:,:,2) .* hvpict .* hcpict;
hhkvpict(:,:,3) = hhkpict(:,:,3) .* hvpict .* hcpict;
hhkvpict(:,:,4) = hhkpict(:,:,4) .* hvpict .* hcpict;
hhkvpict(:,:,5) = hhkpict(:,:,5) .* hvpict .* hcpict;

hkvpict(:,:,1) = hkpict(:,:,1).*(1-hvpict) .* hcpict;
hkvpict(:,:,2) = hkpict(:,:,2) .* hvpict .* hcpict; 
hkvpict(:,:,3) = hkpict(:,:,3) .* hvpict .* hcpict;
hkvpict(:,:,4) = hkpict(:,:,4) .* hvpict .* hcpict;
hkvpict(:,:,5) = hkpict(:,:,5) .* hvpict .* hcpict;


figure;
imshow( fpict );
drawnow;
figure;
imshow( ffkvpict(:,:,2:4) ); % face
drawnow;
figure;
imshow( fkvpict(:,:,2:4) );
drawnow;

figure;
imshow( hpict );
drawnow;
figure;
imshow( hfkvpict(:,:,2:4) );
drawnow;
figure;
imshow( hhkvpict(:,:,2:4) );
drawnow;
figure;
imshow( hkvpict(:,:,2:4) );
drawnow;


else
  load typeGMM_prog
end


graf = [
  [0 0 0];
  [1 0 0];
  [0 1 0];
  [0 1 1];
  [0 0 0];
]';


figure; imshow( fpict );
figure; imshow( type_pict_vis( ffkvpict, graf ) );
figure; imshow( type_pict_vis( fkvpict, graf ) );

figure; imshow( hpict );
figure; imshow( type_pict_vis( hfkvpict, graf ) );
figure; imshow( type_pict_vis( hhkvpict, graf ) );
figure; imshow( type_pict_vis( hkvpict, graf ) );



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
return;

% classify!
pict = pict_read( pictname );
figure;
imshow( pict );
drawnow;

jpict = jnd_pict( pict );


vpict = pict_prob_visible( pict, samp, rad, jpict );

kpict = type_prob_pict( pict, classifier, rad, jpict );

akpict = type_prob_pict( pict, aclassifier, rad, jpict );

kvpict(:,:,1) = kpict(:,:,1).*(1-vpict);
kvpict(:,:,2) = kpict(:,:,2).*vpict;
kvpict(:,:,3) = kpict(:,:,3).*vpict;
kvpict(:,:,4) = kpict(:,:,4).*vpict;
kvpict(:,:,5) = kpict(:,:,5).*vpict;
  
akvpict(:,:,1) = akpict(:,:,1).*(1-vpict);
akvpict(:,:,2) = akpict(:,:,2).*vpict;
akvpict(:,:,3) = akpict(:,:,3).*vpict;
akvpict(:,:,4) = akpict(:,:,4).*vpict;
akvpict(:,:,5) = akpict(:,:,5).*vpict;

figure;
imshow( kvpict(:,:,2:4) );

figure;
imshow( akvpict(:,:,2:4) );



nkpict = type_prob_pict( pict, nclassifier, rad, jpict );

nkvpict(:,:,1) = nkpict(:,:,1).*(1-vpict);
nkvpict(:,:,2) = nkpict(:,:,2).*vpict;
nkvpict(:,:,3) = nkpict(:,:,3).*vpict;
nkvpict(:,:,4) = nkpict(:,:,4).*vpict;
nkvpict(:,:,5) = nkpict(:,:,5).*vpict;

figure;
imshow( nkvpict(:,:,2:4) );



return;

