close all;
clear all;

Nsample = 16;
rad = 3;
load trainingdata;

featuredata = type_feature_data_make( trainingdata, Nsample );
classifier = type_classifier_make( featuredata, rad );

dim = size( featuredata.feature, 1);

% partition _given_ feature data after projection on to visible axis
mask = featuredata.type == 1;
fnv = featuredata.feature( dim, mask ); % not-visible features
fv = featuredata.feature( dim, ~mask ); % visible features

% sample from _full distribution in classifier_, then project to visible axis
m = round( classifier.prior*size( featuredata.feature, 2) );
sv = [];
snv = [];
for i = 1:size(classifier.gmm,2)
  dummy = GMM_sample( classifier.gmm{i}, m(i) );
    if i == 1
      snv = dummy( dim,:);
    else
      sv = [sv dummy(dim,:)];
  end
end

% make a histogram for each sample set
Nbin = round(size( featuredata.feature,2)/30);
% lo = min( [fnv, fv, snv, sv] );
% The _sampled_ data can extend beyond physically reliasable limits.
% For data _synthesis_ this is a problem, because it means that "illegal" data can be produced.
% For data _analysis_ this is not a problem, provided that the distributions in the "leagal" volume is well modlled.
lo = 0; % set lo to be _physical_ limit
hi = max( [fnv, fv, snv, sv]  );
dx = (hi-lo)/(Nbin-1);
x = lo:dx:hi;

hfnv = hist( fnv, x ); % given data histogram
hfv = hist( fv, x );
pfnv = hfnv ./ (hfnv + hfv); % turn into a probability field
pfv = hfv ./ max( eps, hfnv + hfv );

hsnv = hist( snv, x ); % sampled data histogram
hsv = hist( sv, x );
psnv = hsnv ./ (hsnv + hsv);
psv = hsv ./ (hsnv + hsv );

% make a model
mu = 0.4:0.05:0.7;
sigma = 0.2:0.05:0.5;
clear d;
%[u,d] = fminsearch( @type_visible_test_func, [mu;sigma], [], x, pfv );
for i = 1:length(mu)
  for j = 1:length(sigma)
    z = (erf( (x-mu(i)) / sigma(j) ) + 1)/2;
    d(i,j) = norm(z - pfv);
  end
end
dmin = min(min(d));
[i,j] = find( d == dmin );
mu = mu(i);
sigma = sigma(j);
z = (erf( (x-mu) / sigma ) + 1)/2;

% visualise
close
mask = x <= 2;
figure;
hold on;
h1 = plot( x(mask), pfnv(mask), 'b' );
h2 = plot( x(mask), pfv(mask), 'r' );
%h3 = plot( x(mask), psnv(mask), 'r' );
%h4 = plot( x(mask), psv(mask), 'b');
h5 = plot( x(mask), z(mask), 'k' );

msg = sprintf('fitted error function \\mu= %4.2f, \\sigma = %4.2f', mu, sigma);
legend( [h1,h2,h5], 'non-visible training data', 'visible training data', msg);

%legend( [h1,h2,h3,h4,h5], 'non-visible training data', 'visible training data', ...
%  'non-visible sampled data', 'visible sampled data', 'error function');
xlabel( 'mean jnd over disc: \Delta');
ylabel(' probability of being visible: p(\Delta)');
