function g = Gaussian2D( rows, cols, r0,c0, ysigma,xsigma, ang )

% g = Gaussian2D( rows,cols, rorg,corg, xsigma,ysigma, ang )
%   Returns an eliptical Gaussian with size (rows,cols), centre (rorg,corg),
%   and variance (xsigma, ysigma) over principal axes, which are oriented
%   at angle ang to standard x,y axes.
% pre:
%   rows is the number of rows in the output picture.
%   cols is the number of columns in the output picture.
%   rorg is the origin of the Gaussian over the rows.
%   corg is the origin of the Gaussian over the columns.
%   ysigma is the variance of the Gaussian over the ordinate (rows).
%   xsigma is the variance of the Gaussian over the absicca (cols).
%   ang is the angle of the absicca to columns, in radian.
% post;
%   g is a 2D Gaussian.
% Notes:
%   There is no height scaling of the Gaussian
% See Also;

[u,v] = meshgrid( [1:cols]-c0, [1:rows]-r0 );

sang = sin(ang);
cang = cos(ang);

x = u*cang - v*sang;
y = u*sang + v*cang;

g = exp( -0.5*( (x.^2)/xsigma^2 + (y.^2)/(ysigma^2) ) );

return;
