function g = Gaussian2D_deriv( x,y, a,b, derivx, derivy, norm )

% g = Gaussian2D_deriv( x,y, a,b, derivx, derivy, norm )
%
% Compute a 2D Gaussian derivative at sample points (x,y)
% The basic Gaussian is
%     exp( -0.5*((x/a).^2 + (y/b).^2)  )
% The standard deviations, a,b, are scalars. All input values must be real.
% The length of each input vector, x,y,  must be at least 2.
% pre:
%    x - regularly spaced sample points, dx = x(i+1)-x(i),  dx > 0, for all i in {1, length(x)-1}
%    y - as for x: x and y are vectors - they are NOT made with meshgrid.
%   a - the standard deviation over x, a scalar.
%   b - the standard deviation over y, a scalar
%   derivx, derivy - (optional) the derivatives over x and y, compute indexed as follows:
%     0  zeroth derivative (ie. the Gaussian), the default
%     1  first derivative
%     2  second derivative
%     3  third derivative
%     eithe BOTH must be present, or neither
%   norm - (optional) the normal to use, can supplied only if deriv is supplied.
%     0  infinity norm
%     1  L_1 norm (default)
%     2  L_2 norm
% post:
%  g - the Gaussian derivative

% nargin must be 4, 6, or 7.
switch nargin
case 4,
  derivx = 0;
  derivy = 0;
  norm = 1;
case 6,
  norm = 1;
case 7,
otherwise,
  % error in number of input arguments
  g = [];
  return;
end

g = Gaussian1D_deriv( y, b, derivy, norm )' * Gaussian1D_deriv( x, a, derivx, norm );

return;
